from typing import Type

from omnibridge.wrappers.wrapper_interfaces.file_generating_model_wrapper import FileGenModelWrapper

from omnibridge.models_units.base_model_unit import BaseModelUnit
from omnibridge.models_units.models_io.base_model_io import ModelIO
from omnibridge.models_units.models_io.base_model_io import TextualIO, FileIO


class ImageGenModel(BaseModelUnit):
    def __init__(self, image_gen_model_wrapper: FileGenModelWrapper):
        self.wrapper = image_gen_model_wrapper

    def can_process_type(self, model_input_type: Type[ModelIO]) -> bool:
        return issubclass(model_input_type, TextualIO)

    def produced_type(self) -> Type[FileIO]:
        return FileIO

    def process(self, model_input: TextualIO) -> FileIO:  # type: ignore[override]
        if not self.can_process(model_input):
            raise Exception(f"Cannot process input of type: {type(model_input)}")

        prompt = model_input.get_text()
        response = self.wrapper.prompt_and_generate_files(prompt)
        return response  # type: ignore[no-any-return]



from typing import Any

from omnibridge.flows.sequential_flow import SequentialFlow
from omnibridge.models_units.textual_model_unit import TextualModel
from omnibridge.models_units.image_gen_model_unit import ImageGenModel
from omnibridge.wrappers.wrapper_interfaces.textual_model_wrapper import TextualModelWrapper
from omnibridge.wrappers.wrapper_interfaces.file_generating_model_wrapper import FileGenModelWrapper
from omnibridge.models_units.models_io.base_model_io import TextualIO


class MockedGPTWrapper(TextualModelWrapper):
    def prompt_and_get_response(self, prompt: str) -> str:
        return "mocked response"


class MockedDalleWrapper(FileGenModelWrapper):
    def prompt_and_generate_files(self, prompt: str) -> Any:
        return "mocked path of file"


if __name__ == '__main__':
    seq_flow = SequentialFlow()
    chatgpt_model = TextualModel(MockedGPTWrapper())
    dalle_model = ImageGenModel(MockedDalleWrapper())
    seq_flow.add(chatgpt_model)
    seq_flow.add(dalle_model)
    flow_input = TextualIO("flow input")
    result = seq_flow.run_flow(flow_input)
    print(result)



from abc import ABC


class ModelIO(ABC):
    pass


class TextualIO(ModelIO):
    def __init__(self, text: str):
        self.text = text

    def get_text(self) -> str:
        return self.text


class FileIO(ModelIO):
    pass



from typing import Type

from omnibridge.wrappers.wrapper_interfaces.textual_model_wrapper import TextualModelWrapper

from omnibridge.models_units.base_model_unit import BaseModelUnit
from omnibridge.models_units.models_io.base_model_io import ModelIO
from omnibridge.models_units.models_io.base_model_io import TextualIO


class TextualModel(BaseModelUnit):
    def __init__(self, textual_model_wrapper: TextualModelWrapper):
        self.wrapper = textual_model_wrapper

    def can_process_type(self, model_input_type: Type[ModelIO]) -> bool:
        return issubclass(model_input_type, TextualIO)

    def produced_type(self) -> Type[TextualIO]:
        return TextualIO

    def process(self, model_input: TextualIO) -> TextualIO:  # type: ignore
        if not self.can_process(model_input):
            raise Exception(f"Cannot process input of type: {type(model_input)}")

        prompt = model_input.get_text()
        response = self.wrapper.prompt_and_get_response(prompt=prompt)
        textual_output = TextualIO(response)
        return textual_output



from fastapi import APIRouter
from omnibridge.api.app.schemas.ai_model import AIModel
from pathlib import Path
from typing import Any, List
import json

router = APIRouter()

@router.get("/models", response_model=List[AIModel])
def get_models() -> Any:
    models_path = Path(__file__).parents[2] / 'data/ai_models_metadata.json'

    with open(models_path, 'r') as f:
        models = json.load(f)

    return models


from typing import List

from omnibridge.models_units.base_model_unit import BaseModelUnit
from omnibridge.models_units.models_io.base_model_io import ModelIO


class SequentialFlow:
    def __init__(self) -> None:
        self.models: List[BaseModelUnit] = []

    def is_valid_addition(self, new_model: BaseModelUnit) -> bool:
        if len(self.models) == 0:
            return True

        last_unit = self.models[-1]
        if new_model.can_process_type(last_unit.produced_type()):
            return True

        return False

    def add(self, new_model: BaseModelUnit) -> None:
        if not self.is_valid_addition(new_model):
            raise TypeError("Cannot add to this flow a model of this type.")

        self.models.append(new_model)

    def run_flow(self, flow_input: ModelIO) -> ModelIO:
        if len(self.models) == 0:
            raise RuntimeError("Cannot run empty flow.")

        if not self.models[0].can_process(flow_input):
            raise TypeError("Cannot run flow with input of this type.")

        model_input = flow_input
        for model in self.models:
            model_output = model.process(model_input)
            model_input = model_output

        return model_output
